unit Test_uBUGTalk;

interface

uses
  // DUnit
  TestFramework,
  TestExtensions,    // <-- add
  GUITesting,        // <-- add
  // Delphi
  SysUtils, Controls,
  // Application
  uBUGTalk, uBUGTalk_Report;

type
  CRACK_TfmBUGTalk = class(TfmBUGTalk);

//  Check_TfmBUGTalk = class(TTestCase)  <-- alter
  Check_TfmBUGTalk = class(TGUITestCase)
  private
    FTfmBUGTalk : TfmBUGTalk;
    procedure CheckSizeAndShape(const aComponent:TControl;
               aTop,aLeft,aHeight,aWidth: Integer);
  public
    procedure SetUp;  override;
    procedure TearDown; override;
  published
    // GUI
    procedure ShowModal;     // scaffold method - turn off for automated testing
    procedure Verify_InformalAddress;
    procedure Verify_Show;
    procedure Verify_btnInformalClick;
    procedure Verify_SizeAndShape;
    procedure Verify_ActiveAndTab;
    // Helper
    procedure Verify_CompareLines;
    // Web
    procedure Verify_HTML;
    // Database
    procedure Verify_Data;

  end;

  Check_TfmBUGTalkReport = class(TTestCase)
  private
    FTfmBUGTalkReport : TfmBUGTalkReport;
  public
    procedure SetUp;  override;
    procedure TearDown; override;
  published
    // Reports
    procedure PreviewReport; // scaffold method - turn off for automated testing
    procedure Verify_Components;
    procedure Verify_Output;
  end;

  function CompareLines(const aLines1, aLines2: string; out aLineNo : Integer): Boolean;


const
  gPause = 1000;



implementation

uses
  // Delphi
  Dialogs, Graphics, Forms, Classes, QuickRpt, QRPrntr, Math;


{ Check_TfmBUGTalk }

procedure Check_TfmBUGTalk.SetUp;
  begin
    inherited;
    FTfmBUGTalk         := TfmBUGTalk.Create(nil);
    FTfmBUGTalk.Color   := clYellow;
    FTfmBUGTalk.Caption := 'This Form is being tested';
    GUI                 := FTfmBUGTalk;
    ActionDelay         := gPause;
  end;

procedure Check_TfmBUGTalk.TearDown;
  begin
    GUI := nil;
    FreeAndNil(FTfmBUGTalk);
    inherited;
  end;

procedure Check_TfmBUGTalk.ShowModal;
  begin
    FTfmBUGTalk.ShowModal
  end;

procedure Check_TfmBUGTalk.Verify_InformalAddress;
  begin
    CheckEquals('Jane Smith',InformalAddress('Miss Smith','Jane'));
  end;

procedure Check_TfmBUGTalk.Verify_Show;
  begin
    Show(True);
    Check(FTfmBUGTalk.Visible = True);
  end;

procedure Check_TfmBUGTalk.Verify_btnInformalClick;
  begin
    FTfmBUGTalk.Show;
    Application.ProcessMessages;

    CheckEquals('Mr Rye',FTfmBUGTalk.ledFormal.Text);
    CheckEquals('Adrian',FTfmBUGTalk.ledFirstName.Text);
    CheckVisible('btnInformal');
    ClickLeftMouseButtonOn(FTfmBUGTalk.btnInformal);
    CheckEquals('Adrian Rye',FTfmBUGTalk.ledInFormal.Text);

    Sleep(gPause);
    Hide;
  end;

procedure Check_TfmBUGTalk.Verify_SizeAndShape;
  begin
    CheckSizeAndShape(FTfmBUGTalk.bevAddress,8,8,57,521);
    CheckSizeAndShape(FTfmBUGTalk.ledFormal,32,24,21,121);
  end;

procedure Check_TfmBUGTalk.CheckSizeAndShape(const aComponent: TControl;
  aTop,aLeft,aHeight,aWidth: Integer);
  begin
    CheckVisible(aComponent);
    CheckEquals(aTop,    aComponent.Top,    aComponent.Name + ' top wrong');
    CheckEquals(aLeft,   aComponent.Left,   aComponent.Name + ' left wrong');
    CheckEquals(aHeight, aComponent.Height, aComponent.Name + ' height wrong');
    CheckEquals(aWidth,  aComponent.Width,  aComponent.Name + ' width wrong');
  end;

procedure Check_TfmBUGTalk.Verify_ActiveAndTab;
  begin
    FTfmBUGTalk.Show;
    Application.ProcessMessages;

    CheckFocused(FTfmBUGTalk.btnInformal);
    Tab;
    CheckFocused(FTfmBUGTalk.ledFormal);
    Tab;
    CheckFocused(FTfmBUGTalk.ledFirstName);
    Tab;
    CheckFocused(FTfmBUGTalk.ledInformal);

    Sleep(gPause);
    Hide;
  end;

procedure Check_TfmBUGTalk.Verify_CompareLines;
  var
    theString1 : string;
    theString2 : string;
    outLine    : Integer;
  begin
    // when developing the checks I usually let them fail and then add a 'not' if
    // the failure response is the one I wanted.

    theString1 := 'A String';
    theString2 := 'A String';
    Check(CompareLines(theString1,theString2, outLine),'a ' + IntToStr(outLine));

    theString2 := 'B String';
    Check(not CompareLines(theString1,theString2, outLine),'b ' + IntToStr(outLine));

    theString1 := 'A String,B String,C String';
    theString2 := 'A String,B String,C String';
    Check(CompareLines(theString1,theString2, outLine),'c ' + IntToStr(outLine));

    theString2 := 'A String,D String,C String';
    Check(not CompareLines(theString1,theString2, outLine),'d ' + IntToStr(outLine));

    theString1 := Format('%s%s%s%s%s',['A String',#13,'B String',#13,'C String']);
    theString2 := Format('%s%s%s%s%s',['A String',#13,'B String',#13,'C String']);
    Check(CompareLines(theString1,theString2, outLine),'e ' + IntToStr(outLine));

    theString2 := Format('%s%s%s%s%s',['A String',#13,'D String',#13,'C String']);
    Check(not CompareLines(theString1,theString2, outLine),'f ' + IntToStr(outLine));
    CompareLines(theString1,theString2, outLine);
    Check(outLine = 1,'g ' + IntToStr(outLine));
  end;


procedure Check_TfmBUGTalk.Verify_HTML;
  var
    outLine : Integer;
  begin
    // Note: the HTML produced does containg returns, and will produce multiple
    //       lines in a stringlist.
    if FileExists('BugTest1.htm') then begin
      with TStringList.Create do begin
        try
          LoadFromFile('BugTest1.htm');

          Check(CompareLines(Text, FTfmBUGTalk.dstpData.Content, outLine),
                            'HTML has changed at line '+ IntToStr(outLine));
        finally
          Free
        end;
      end;
    end else begin
      Fail('BugTest1.htm missing');
    end;
  end;

procedure Check_TfmBUGTalk.Verify_Data;
  var
    outLine : Integer;
  begin
    // Note: the XML produced does not containg returns, and will not produce multiple
    //       lines in a stringlist. Compare lines could be improved by using the
    //       delimited text property and setting a delimiter.
    if FileExists('TestData1.xml') then begin
      with TStringList.Create do begin
        try
          LoadFromFile('TestData1.xml');

          Check(CompareLines(Text, FTfmBUGTalk.cdsData.XMLData, outLine),
                         'Data has changed at line '+ IntToStr(outLine));
        finally
          Free
        end;
      end;
    end else begin
      Fail('TestData1.xml');
    end;
  end;


{ Check_TfmBUGTalkReport }

procedure Check_TfmBUGTalkReport.SetUp;
  begin
    inherited;
    FTfmBUGTalkReport := TfmBUGTalkReport.Create(nil);
  end;

procedure Check_TfmBUGTalkReport.TearDown;
  begin
    FreeAndNil(FTfmBUGTalkReport);
    inherited;
  end;

procedure Check_TfmBUGTalkReport.PreviewReport;
  begin
    FTfmBUGTalkReport.repBUGTalk.Preview;
  end;

procedure Check_TfmBUGTalkReport.Verify_Components;
  begin
    // individual items
    CheckEquals('BUGTalk Test Report', FTfmBUGTalkReport.lblTitle.Caption, 'Title Changed');
    CheckEquals(Ord(taCenter), Ord(FTfmBUGTalkReport.lblTitle.Alignment), 'Alignment Wrong');
    CheckEquals(24, FTfmBUGTalkReport.lblTitle.Font.Size,'Font size wrong');
    // expected bands
    CheckEquals(4, FTfmBUGTalkReport.repBUGTalk.BandList.Count, 'Band count changed');
    // expected standard bands
    Check(TQRCustomBand(FTfmBUGTalkReport.repBUGTalk.BandList.First).BandType = rbTitle, 'No/Wrong Title');
    Check(TQRCustomBand(FTfmBUGTalkReport.repBUGTalk.BandList.Last).BandType  = rbPageFooter, 'No/Wrong Page Footer');
  end;

procedure Check_TfmBUGTalkReport.Verify_Output;
  var
    test1 : TStringList;
    test2 : TStringList;
  begin
    // This test requires that the defaut printer is a generic text printer
    // printing to a file. It is not a good automated solution because it
    // requires a filename each time it is run.
    FTfmBUGTalkReport.repBUGTalk.Prepare;
    FTfmBUGTalkReport.repBUGTalk.Print;

    if FileExists('BUGTest1.prn') then begin
      if FileExists('BUGTest2.prn') then begin
        test1 := TStringList.Create;
        test2 := TStringList.Create;
        try
          test1.LoadFromFile('BUGTest1.prn');
          test2.LoadFromFile('BUGTest2.prn');
          Check(CompareStr(test1.Text,test2.Text)= 0,'The Report has changed');
        finally
          FreeAndNil(test1);
          FreeAndNil(test2);
        end;
      end else begin
        Fail('File Missing BUGTest2.prn');
      end;
    end else begin
      Fail('File Missing BUGTest1.prn');
    end;
  end;

function CompareLines(const aLines1, aLines2: string; out aLineNo : Integer): Boolean;
  var
    theLines1 : TStringList;
    theLines2 : TStringList;
    i         : Integer;
  begin
    aLineNo   := 0;
    theLines1 := TStringList.Create;
    theLines2 := TStringList.Create;
    try
      theLines1.Text := aLines1;
      theLines2.Text := aLines2;
      if theLines1.Count = theLines2.Count then begin
        for i := 0 to pred(theLines1.Count) do begin
          if CompareStr(theLines1[i],theLines2[i]) <> 0 then begin
            aLineNo := i;
            Result  := False;
            Exit;
          end;
        end;
        Result := True;
      end else begin
        Result := False;
      end;
    finally
      theLines1.Free;
      theLines2.Free;
    end;
  end;



initialization;
  begin
    RegisterTest('GUI Tests', Check_TfmBUGTalk.Suite);
    RegisterTest('Report Tests', Check_TfmBUGTalkReport.Suite);
  end;

end.

